#!/bin/bash

# Check for presence of a suitable Java JDK. Script is called
# during installation of heliumv-base package, and
# by startup scripts of heliumv-app and heliumv-api.
#
#	Arguments:
#		[1] (optional) ... full path of defaults file
#
#	Outputs:
#		nothing except error/warning messages
#
#	Returns
#		0 ... if everything OK
#		1 ... on any kind of error
#
# Expected behaviour:
#
#	*	reads defaults file. Not exist -> Warning (don't bail out here!)
#	*	checks for JAVA_HOME variable: is set, dir exists, java executable
#		exists, is a JDK, java version 1.7 or greater, is original 'Java(TM)'
#		(last check currently deactivated)
#	*	all checks passed -> OK, exit 0, else...
#	*	list alternatives, loop over all until one passes all checks
#	*	one found -> edit defaults file, exit 0
#	*	none found -> print ERROR, exit 1
#
# When arg[1] is not given, script only searches for a suitable JDK
# and does not read or write any config file.
 
# set to '/dev/stderr' to get debug output
OUTPUT=/dev/null

# Function checks JAVA validity and returns error text or nothing.
# Arg is a directory, perhaps suitable as JAVA_HOME
check_valid_java()
{
	local JAVA_DIR=$1
	
	# This check should fire after a Java upgrade where the old JDK was
	# entirely replaced (removed)
	if [ ! -d ${JAVA_DIR} ] ; then
		echo "Java Home '${JAVA_DIR}' does not exist or is not a directory"
		return
	fi
	
	local JAVA_BINARY=${JAVA_DIR}/bin/java
	# Check against GCJ masking itself as 'java'. Note 'java' executable
	# can be in different locations of a JRE/JDK distribution
	if [ ! -x ${JAVA_BINARY} ] ; then
		JAVA_BINARY=${JAVA_DIR}/jre/bin/java
		if [ ! -x ${JAVA_BINARY} ] ; then
			echo "Java Home '${JAVA_DIR}' Perhaps not a JRE/JDK home"
			return
		fi
	fi
	
	# Check for JDK (as opposed to JRE)
	if [ ! -x ${JAVA_DIR}/bin/jar ] ; then
		echo "Java Home '${JAVA_DIR}' perhaps not a JDK home"
		return
	fi
	
	# Only 1.7.xxx is allowed.
	local JRE_VERS=`${JAVA_BINARY} -version 2>&1`
	local JAVA_VERS=`echo "${JRE_VERS}" | head -n 1 | awk '{print $NF;}' | tr -d '"'`
	if ! expr "${JAVA_VERS}" : '1.7' >/dev/null ; then
		echo "Java version '${JAVA_VERS}' not suitable for this application"
		return
	fi
	
	# Check for 'genuine' (Sun resp. Oracle) Java (hopefully never needed)
#	local JAVA_PRODUCT=`echo "${JRE_VERS}" | head -n 2 | tail -n 1`
#	local JPL=`echo "${JAVA_PRODUCT}" | tr 'A-Z' 'a-z'`
#	if ! expr "${JPL}" : '.*java(tm)' >/dev/null; then
#		local JPS=`echo "${JAVA_PRODUCT}" | sed 's/ (.*//'`
#		echo "Java product '${JPS}' not suitable for this application"
#		return
#	fi
}

CONFFILE=$1
JAVA_HOME=
if [ -n "${CONFFILE}" ] ; then
	if [ -e ${CONFFILE} ] ; then
		. ${CONFFILE}
	else
		echo "WARNING: config file '${CONFFILE}' does not exist" >&2
	fi
fi

if [ -n "${JAVA_HOME}" ] ; then
	ERR=`check_valid_java ${JAVA_HOME}`
	if [ -z "${ERR}" ] ; then
		exit 0
	fi
	
	echo "WARNING: ${ERR}" >&2
	JAVA_HOME=
else
	if [ -n "${CONFFILE}" ] ; then
		echo "WARNING: config file '${CONFFILE}' does not contain JAVA_HOME setting"
	fi
fi

TMPF=/tmp/check_java_$$.tmp
trap "rm -f ${TMPF}" EXIT
update-java-alternatives --list | awk '{print $NF;}' >${TMPF}
while read JAVA_BIN; do
	echo "Checking JAVA_BIN = '${JAVA_BIN}' ..." >${OUTPUT}
	ERR=`check_valid_java ${JAVA_BIN}`
	if [ -n "${ERR}" ] ; then
		echo "WARNING: ${ERR}" >${OUTPUT}
	else
		JAVA_HOME=${JAVA_BIN}
	fi
done <${TMPF}

if [ -z "${JAVA_HOME}" ] ; then
	echo "ERROR: Can't find a suitable JDK" >&2
	exit 1
fi

# now write config
echo "Found suitable JDK in '${JAVA_HOME}'" >&2

# no config file given -> don't create/change one.
[ -z "${CONFFILE}" ] && exit 0

echo "Changing config file '${CONFFILE}' now." >&2
(
	cat ${CONFFILE} 2>/dev/null | grep -vE '^JAVA_HOME='
	echo "JAVA_HOME=${JAVA_HOME}"
) >${CONFFILE}.new

mv -f ${CONFFILE}.new ${CONFFILE}
exit 0

